#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"
#define VENDOR_RQ_SET_LED    0
#define VENDOR_RQ_GET_SWITCH 1

/* ------------------------------------------------------------------------- */
/* ---------------------------- Utility Functions -------------------------- */
/* ------------------------------------------------------------------------- */

char in_c(char pin)
{
    // read logic level of port-C's pin
    DDRC &= ~(1<<pin);
    return ((PINC & (1<<pin))>>pin);    
} 

void out_c(char pin, char val)
{
    // write logic level to port-C's pin
    DDRC |= 1<<pin;
    if(val)
        PORTC ^= 1<<pin;
    else
    	PORTC &= 0<<pin;
}
void out_b(char pin, char val)
{
    // write logic level to port-B's pin
	
    DDRB |= 1<<pin;
    if(val)
        PORTB ^= 1<<pin;
    else
    	PORTB &= 0<<pin;
}

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
    usbRequest_t *rq = (void *)data;
    static uchar dataBuffer[1];  /* must stay valid when usbFunctionSetup returns */

    if (rq->bRequest == VENDOR_RQ_SET_LED)
    {        
		uint8_t led_no  = rq->wValue.bytes[0];
        uint8_t led_val = rq->wValue.bytes[1];
		if(led_no >= 3)
		{
			led_no = led_no-3;
			out_b(led_no, led_val);
			return 0;
		}
        out_c(led_no, led_val);
        return 0;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCH)
    {
        dataBuffer[0] = !in_c(3);
        usbMsgPtr = dataBuffer;
        return 1;
    }
    return 0;   /* default for not implemented requests: return no data back to host */
}


/* ------------------------------------------------------------------------- */

int main(void)
{
    DDRC  = 0b00000111;  // PC0..PC2 = Output, PC3..PC5 = Input
    PORTC = 0b00000000;  // PC7..PC0 = 0
    //PORTC |= (1 << 3);   // Enable pull-up at PC3
    DDRB  = 0b00111111;
    PORTB = 0b00000000;
	int count=0;
    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    _delay_ms(300);
    usbDeviceConnect();
    sei();
    for(;;){                /* main event loop */
        usbPoll();
		while((PORTB == 0b00111111) && (PORTC == 0b00000111) && count <= 10)
		{		
		PORTC = 0b00000000;  // ส่งลอจิก 0 ไปที่ขา PD3
        _delay_ms(1);		
		PORTB = 0b00000010;  // ส่งลอจิก 1 ไปที่ขา PD3
        _delay_ms(50);
        PORTB = 0b00000001;  // ส่งลอจิก 0 ไปที่ขา PD3
        _delay_ms(50);
		PORTB = 0b00001000;  // ส่งลอจิก 0 ไปที่ขา PD3
        _delay_ms(50);
		PORTB = 0b00000100;  // ส่งลอจิก 1 ไปที่ขา PD3
        _delay_ms(50);
        PORTB = 0b00100000;  // ส่งลอจิก 0 ไปที่ขา PD3
        _delay_ms(50);
		PORTB = 0b00010000;  // ส่งลอจิก 0 ไปที่ขา PD3
        _delay_ms(50);
		PORTB = 0b00000000;  // ส่งลอจิก 0 ไปที่ขา PD3
        _delay_ms(1);
		PORTC = 0b00000100;  // ส่งลอจิก 1 ไปที่ขา PD3
        _delay_ms(50);
        PORTC = 0b00000010;  // ส่งลอจิก 0 ไปที่ขา PD3
        _delay_ms(50);
		PORTC = 0b00000001;  // ส่งลอจิก 0 ไปที่ขา PD3
        _delay_ms(50);
		PORTC = 0b00000000;  // ส่งลอจิก 0 ไปที่ขา PD3
        _delay_ms(1);
		PORTB = 0b00111111;  // ส่งลอจิก 0 ไปที่ขา PD3
        _delay_ms(1);
		PORTC = 0b00000111;  // ส่งลอจิก 0 ไปที่ขา PD3
        _delay_ms(1);
		
		count = count+1;
		}
		if(count>10)		
		{	PORTB = 0b00000000;
			PORTC = 0b00000000;
			count = 0;
		}
     }	

	
    return 0;
}

/* ------------------------------------------------------------------------- */
